#!/usr/bin/perl
#
# Generate a new .signature file from a .id file and one of the quotes from
# the sigs directory.
#
# (C)1999 Michael J. Hammel
# Free for public use and redistribution, as long as my copyright remains
# with this file.
#-------------------------------------------------------------------------
use Getopt::Long;
use Cwd;

#-------------------------------------------------------------------------
# Set the defaults for 
# SIGFILE: file to write the signature to
# IDFILE: the id file (ie the raw sig file)
# QUOTEDIR: quote files top level directory
# FORMID: which form type to use.
#
my $SIGFILE  = "$ENV{'HOME'}/.signature";
my $IDFILE1  = "$ENV{'HOME'}/src/MuseSig/form1-id";
my $IDFILE2  = "$ENV{'HOME'}/src/MuseSig/form2-id";
my $IDFILE   = $IDFILE1;
my $QUOTEDIR = "$ENV{'HOME'}/src/MuseSig/sigs";
my $FORMID   = "form1";
my $formtype = 1;

#-------------------------------------------------------------------------
#  Display the currently configured id files.
sub showids {
	print "ID1 file: $IDFILE1\n";
	open (FD, $IDFILE1);
	@data = <FD>;
	close(FD);
	print @data;
	print "\n";

	print "ID2 file: $IDFILE2\n";
	open (FD, $IDFILE2);
	@data = <FD>;
	close(FD);
	print @data;
	print "\n";

	exit(0);
}

#-------------------------------------------------------------------------
# Display the README
sub verbose {
$info =<<EOF;
make-sig.pl:  Generate a new .signature file from templates.
---------------------------------------------------------------

This simple perl script takes input from three sources (two at a time,
actually, but 3 possible sources) and generates a text based .signature
file.  It's use is mostly asthetic.  Signature files are kind of outdated
in this world of HTML email, although if you remove dependencies on
position in fixed width fonts they may be just fine.

To use make-sig.pl, do the following:

1. Edit form1-id and form2-id with your contact information.
2. run make-sig.pl (or "make-sig.pl -?" to get usage information)

Form1-id is used with the quotes under sigs/form1.   Form1 is specific for
use with fixed with fonts - variable width fonts mess up the positioning.
Form1 quotes are positioned to the right of the form1-id text in your
.signature file.

Form2-id is used with the quotes under sigs/form2.  Form2 doesn't care
about font type, so you can put anything in the form2-id you want and the
format of the quote files is format independent as well.  Form2 quotes are
appened after your form2-id text in your .signature file.

To add new quotes to form1, copy the top level "template" into the
sigs/form1 directory using any filename.  Then edit it to fit the space
between the two vertical bars.  Remove the bars and header before saving.
These quote files should be 4 lines long - no more, no less.  Use blank
lines for lines with no text.

To add new quotes to form2, just create a file in the sigs/form2 directory
and fill it with anything you want.  

That's it.  Pretty simple, but cute.  Add make-sig.pl to your crontab entry
to update your signature every few minutes or so.   Try a line like this
for updates every 15 minutes:

3,18,33,48 * * * * make-sig.pl 2>&1 > /dev/null


Use of make-sig.pl with Evolution and other email composers
------------------------------------------------------------
If your email composer does not use $HOME/.signature as its signature file,
which Evolution does not for example, you can use the --sig <file> option
to specify where to place the signature file.  The only requirement is that
the signature file be allowed to be in plain text format.  The following
example shows how to put a new signature file in one of Evolutions
signature files:

make-sig.pl --sig $HOME/evolution/signatures/signature-0

That's about it.


Have fun.
Michael J. Hammel
mjhammel@graphics-muse.org
EOF
print "$info";
exit(0);
}


#-------------------------------------------------------------------------
#  Explain how to format your id and quote files.
sub showforms {

$info =<<EOF;
There are two formats that you can use to build your signature file.
Both formats limit line lengths to 80 characters.  Both formats allow
id information and quotes to be free formatted, but the first format puts
additional limits on line lengths for id and quotes while the second does not.

The first format is limited to 4 lines and uses the following format 
(the numbers show columns):
         1         2         3         4         5         6         7         8
12345678901234567890123456789012345678901234567890123456789012345678901234567890
================================================================================
Michael J. Hammel           |
The Graphics Muse           |
mjhammel\@graphics-muse.org  |
http://www.graphics-muse.com 
================================================================================
As you can see, your contact information is on the left and the signature
is on the right.  Your signature should take up no more than 29 columns and
4 lines.  The vertical bar (|) is optional.  The text in the quote cannot
be more than 49 characters in width.  This script will take your contact
information (in your id file) and combine it, line for line, with the lines
in the quote file.  This format requires both id file and quote file to
each have 4 lines - no more, no less.

The second format is more flexible.  It allows id and quote files to be as 
many lines as the user chooses.  Additionally, text in both id and quote
files can be free formatted with only an 80 character limit to line
lengths.  A typical id file in this format looks like this:
================================================================================
Michael J. Hammel                               The Graphics Muse 
mjhammel@graphics-muse.org                      http://www.graphics-muse.com
------------------------------------------------------------------------------
================================================================================
The single-dashed line is added by this program to separate the id information
from the quote, which is added below that line.
EOF

	print "$info";
	exit(0);
}

#-------------------------------------------------------------------------
#  Handle command line options
sub usage {
	for($i=0; $i<length($0); $i++) { $pad = "$pad" . " "; }
	print "$0 [ -2 | -r | --sig <file> | --id1 <file> | --id2 <file> |\n" .
		 "$pad   --quote <file> | --showforms | --verbose ] \n" .
			"where \n" .
			"-2                Use form type 2 (defaults to form type 1) \n" .
			"-r                Pick form type randomly \n" .
			"--sig <file>      name of file to write signature to.\n" .
			"                  default: $SIGFILE\n" .
			"--id1 <file>      id file for format 1.\n" .
			"                  default: $IDFILE1\n" .
			"--id2 <file>      id file for format 2.\n" .
			"                  default: $IDFILE2\n" .
			"--quote <file>    name of quote file.\n" .
			"                  default: select quote randomly based on format\n" .
			"--quotedir <dir>  directory where quotes are stored.\n" .
			"                  default: $QUOTEDIR\n" .
			"--showforms       Show the form types.\n" .
			"--showids         Show current id files.\n" .
			"--verbose         Show the README file.\n" .
			"\n" .
			"All file arguments should be fully qualified.\n" .
			"\n" .
			"This script builds a signature file from an idfile and one\n" .
			"of a set of quote files chosen at random or specifically.\n" .
			"The signature file is the file used by your mail composer.\n" .
			"The id file is the template with your name, email address, \n" .
			"and so forth.\n" .
			"\n" .
			"Quotes will be taken from $QUOTEDIR under form1 (if -2 is \n" .
			"specified, then form2). \n" .
			"\n" .
			"You should edit both id file form types with your contact data.\n" .
			"\n" .
			"Use the --showforms option to see the formats that id and quote\n" .
			"files can use.\n" .
			"\n";
	exit(0);
}

$okay = GetOptions ( "sig=s"     => \$SIGFILE,
                     "id1=s"     => \$IDFILE1,
                     "id2=s"     => \$IDFILE2,
                     "quote=s"   => \$QUOTEFILE,
                     "2"         => \$form2,
                     "r"         => \$dorandform,
                     "h|?"       => \$dousage,
                     "showforms" => \$doforms,
                     "showids"   => \$doids,
                     "quotedir=s"=> \$QUOTEDIR,
                     "verbose"   => \$verbose);
if ( !$okay )   { usage(); }
if ( $dousage ) { usage(); }
if ( $doforms ) { showforms(); }
if ( $doids )   { showids(); }
if ( $form2 )   { $formtype = 2; }
if ( $verbose ) { verbose(); }

# If requested, pick format type randomly.
if ( $dorandform )
{
	srand(time|$$);
	$formtype = int(rand(2))+1;
}

# Reset the IDFILE since it may have changed by the command line args.
if ( $formtype == 2 ) { $IDFILE = $IDFILE2; }
else                  { $IDFILE = $IDFILE1; }

# Are we going to use the old format, or the new one?
if ( $formtype == 2 ) { $FORMID = "form2"; }
$TOP = "$QUOTEDIR/$FORMID";


#
# Open the sigs directory and determine how many files there are.
# Use this to get a random number index.
#
opendir(SIGDIR, $TOP);
@allfiles = readdir(SIGDIR);
closedir(SIGDIR);
splice(@allfiles,0,2);
scalar(@allfiles);

# Choose an random index for the quote file.
$rand_index = int(rand(scalar(@allfiles)));


#
# Merge the two files, line by line and then write it 
# out to the signature file.
#
if ( !open(ID, "$IDFILE") )
{
	printf("Can't open ID file: %s\n", $IDFILE);
	exit(1);
}
if ( !open(SIGDATA, "$TOP/$allfiles[$rand_index]") )
{
	printf("Can't open quotes file: %s\n", $TOP/$allfiles[$rand_index]);
	exit(1);
}

#
# Build and output the signature file.
#
open(SIGFILE, ">$SIGFILE");

if ( $formtype == 2 )
{
	# Newer Format - sig across the top line, quote takes next three lines
	while(<ID>)
	{
		printf(SIGFILE "%s", $_);
	}
	while(<SIGDATA>)
	{
		printf(SIGFILE "%s", $_);
	}
}
else
{
	# Original Format
	while(<SIGDATA>)
	{
		push(@quote_text, $_);
	}
	$count = 0;
	while(<ID>)
	{
		chop($_);
		printf(SIGFILE "%s%s", $_, $quote_text[$count++]);
	}
}

close(SIGDATA);
close(ID);
close(SIGFILE);

# Now print it out, just to show what we installed.
open(SIGFILE, "$SIGFILE");
while(<SIGFILE>)
{
	printf("%s", $_);
}
close(SIGFILE);
